#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <gtk/gtk.h>

#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "rr_main.h"
#include "rr_screen.h"

GtkWidget *fileload;
GtkWidget *sizedialog;
GtkWidget *errordialog;

static gfloat   mag_w = 1.0;
static gfloat   mag_h = 1.0;
static gint     rot_val = 0;

void
on_mainwin_destroy                     (GtkObject       *object,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


gboolean
on_screen_expose_event                 (GtkWidget       *widget,
                                        GdkEventExpose  *event,
                                        gpointer         user_data)
{
  return(RRScreenRedraw(widget, event, user_data));
}


void
on_load_ok_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  gchar *fname;
  gboolean  result;

  /* --- եɥλ */
  fname = (gchar*)strdup(gtk_file_selection_get_filename(
		    GTK_FILE_SELECTION(fileload)));

  /* --- λ */
  gtk_widget_destroy(fileload);

  /* --  */
  RRMainIntervalStop();

  /* --- ե͡,ǡɤ߹ */
  result = RRMainSetPicture(fname);
  free(fname);

  /* --- 顼ɽ */
  if (result == FALSE) {
    errordialog = create_error1();
    gtk_label_set_text((GtkLabel*)RRGetParts(RRGetParts(errordialog,"dialog_vbox2"), "errorname"),
		       _("Picture load is failed."));
    gtk_grab_add(errordialog);
    gtk_widget_show (errordialog);
  }

  /* -- ߳ */
  RRMainInterval(1000/4);
}


void
on_loadrequest_activate                (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  fileload = create_fileload();
  gtk_grab_add(fileload);
  gtk_widget_show (fileload);
}


void
on_resizerequest_activate              (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  GtkHScale  *slide;
  GtkEntry   *zoom;
  gchar  text[128];

  sizedialog = create_sizedialog();
  /* 饤ɥСɤߤȤ */
  slide = GTK_HSCALE(lookup_widget(sizedialog, "rot_slide"));
  slide->scale.range.adjustment->value = rot_val;
  /* ΨɤߤȤ */
  zoom = GTK_ENTRY(lookup_widget(sizedialog, "zoom1"));
  snprintf(text, 127, "%1.2f", mag_w);
  gtk_entry_set_text(zoom, text);
  zoom = GTK_ENTRY(lookup_widget(sizedialog, "zoom2"));
  snprintf(text, 127, "%1.2f", mag_h);
  gtk_entry_set_text(zoom, text);
  /* ɽ */
  gtk_grab_add(sizedialog);
  gtk_widget_show(sizedialog);
}


void
on_exit1_activate                      (GtkMenuItem     *menuitem,
                                        gpointer         user_data)
{
  gtk_main_quit();
}


gboolean
on_mainwin_button_press                (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GtkMenu *menu;
  GdkEventButton *event_button;

  if (event->type == GDK_BUTTON_PRESS) {
    event_button = (GdkEventButton *) event;
    if (event_button->button == 1) {
      RRMainDragStart();
      return TRUE;
    }
    if (event_button->button == 3) {
      menu = GTK_MENU (create_menu1());
      g_return_val_if_fail (widget != NULL, FALSE);
      g_return_val_if_fail (event != NULL, FALSE);
      gtk_menu_popup (menu, NULL, NULL, NULL, NULL, 
		      event_button->button, event_button->time);
      return TRUE;
    }
  }
  return FALSE;
}


gboolean
on_mainwin_button_release              (GtkWidget       *widget,
                                        GdkEventButton  *event,
                                        gpointer         user_data)
{
  GdkEventButton *event_button;

  if (event->type == GDK_BUTTON_RELEASE) {
    event_button = (GdkEventButton *) event;
    if (event_button->button == 1) {
      RRMainDragEnd();
      return TRUE;
    }
  }
  return FALSE;
}


gboolean
on_mainwin_motion_notify_event         (GtkWidget       *widget,
                                        GdkEventMotion  *event,
                                        gpointer         user_data)
{
  if (event->type == GDK_MOTION_NOTIFY) {
    RRMainDragMove();
    return TRUE;
  }
  return FALSE;
}


void
on_load_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  /* --- եɥ֥󥻥 */
  gtk_widget_destroy(fileload);
}


void
on_set1_ok_clicked                     (GtkButton       *button,
                                        gpointer         user_data)
{
  GtkHScale  *slide;
  GtkEntry   *zoom;
  gint  r=0;
  gfloat  wz, hz;
  gchar  *text;
  gboolean  result;

  /* --- λ */
  RRMainIntervalStop();
  /* 饤ɥСɤߤȤ */
  slide = GTK_HSCALE(lookup_widget(sizedialog, "rot_slide"));
  r = (gint)(slide->scale.range.adjustment->value);
  /* ΨɤߤȤ */
  zoom = GTK_ENTRY(lookup_widget(sizedialog, "zoom1"));
  text = gtk_entry_get_text(zoom);
  sscanf(text, "%f", &wz);
  zoom = GTK_ENTRY(lookup_widget(sizedialog, "zoom2"));
  text = gtk_entry_get_text(zoom);
  sscanf(text, "%f", &hz);

  /* --- λ */
  gtk_widget_destroy(sizedialog);

  /* ܹ */
  result = RRMainResize(r, wz, hz);
  mag_w = wz;
  mag_h = hz;
  rot_val = r;

  /* -- ߳ */
  RRMainInterval(1000/4);

  /* --- 顼ɽ */
  if (result == FALSE) {
    errordialog = create_error1();
    gtk_label_set_text((GtkLabel*)lookup_widget(errordialog,"errorname"),
		       _("Zoom value is wrong."));
    gtk_grab_add(errordialog);
    gtk_widget_show (errordialog);
  }
}


void
on_set1_cancel_clicked                 (GtkButton       *button,
                                        gpointer         user_data)
{
  /* --- λ */
  gtk_widget_destroy(sizedialog);
}


void
on_error1_ok                           (GtkButton       *button,
                                        gpointer         user_data)
{
  /* --- λ */
  gtk_widget_destroy(errordialog);
}

